#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <portbits.h>
#include "global.h"
#include "rf22.h"
#include <util/delay.h>

typedef struct {
	prog_uchar ifbw;
	prog_uchar rxosr1;
	prog_uchar rxosr0;
	prog_uchar ncoff1;
	prog_uchar ncoff0;
	prog_uchar crgain1;
	prog_uchar crgain0;
} BAUDSET;

BAUDSET baudlist[] PROGMEM={
{0x33,0xD0,0x00,0x9D,0x49,0x00,0xA0},	// 2,4kBaud, 4,8kHz
{0x01,0x83,0xC0,0x13,0xA9,0x00,0x05},	// 2,4kBaud, 36kHz
{0x34,0x68,0x01,0x3A,0x93,0x02,0x78},	// 4,8kBaud, 4,8kHz
{0x04,0x41,0x60,0x27,0x52,0x00,0x0A},	// 4,8kBaud, 45kHz
{0x21,0x64,0x01,0x47,0xAE,0x05,0x21},	// 10kBaud, 5kHz
{0x03,0x90,0x20,0x51,0xEC,0x00,0x2B},	// 10kBaud, 40kHz
{0x11,0x64,0x01,0x47,0xAE,0x05,0x21},	// 20kBaud, 10kHz
{0x04,0xC8,0x00,0xA3,0xD7,0x00,0xA6},	// 20kBaud, 40kHz
{0x01,0x64,0x01,0x47,0xAE,0x05,0x21},	// 40kBaud, 20kHz
{0x05,0x64,0x01,0x47,0xAE,0x02,0x91},	// 40kBaud, 40kHz
{0x0F,0x78,0x01,0x11,0x11,0x04,0x46},	// 100kBaud, 50kHz
{0x0E,0x78,0x01,0x11,0x11,0x00,0xB8}	// 100kBaud, 300kHz
};

static inline unsigned char rf22_trans(unsigned char addr, unsigned char data)
{	CS=0;
	SPDR = addr;
	while(!(SPSR & (1<<SPIF)));
	SPDR = data;
	while(!(SPSR & (1<<SPIF)));
	CS=1;
	return SPDR;
}

void rf22_write(unsigned char addr, unsigned char data)
{
	rf22_trans(128|addr,data);
}

unsigned char rf22_read(unsigned char addr)
{
	return rf22_trans(addr&127,0);
}

void rf22_init(void)
{
	DDRB|=44;
	DDRC|=3;
	SPCR=(1<<SPE)|(1<<MSTR);
	SPSR|=(1<<SPI2X);

	_delay_ms(50);				// wait until POR done
    rf22_write(0x07, 0x80);		// software reset
	_delay_us(500);				// wait until software reset done

    rf22_write(0x05, 0x02);		// valid packed received interrupt on

    rf22_write(0x06, 0x00);		// all interrupts off
    rf22_write(0x07, 0x01);		// operating mode: ready mode
    rf22_write(0x09, 0x7f);		// xtal load capacitance
    rf22_write(0x0A, 0x02);		// uC CLK: 10MHz
    
    rf22_write(0x0b, 0xf2);		// GPIO0: TX_ANT
    rf22_write(0x0c, 0xf5);		// GPIO1: RX ANT
    rf22_write(0x0d, 0x00);		// GPIO2: uC Clock out
    rf22_write(0x0e, 0x00);		
    rf22_write(0x0f, 0x70);		// ADC Input: GND
    rf22_write(0x10, 0x00);		// ADC offset: 0
    rf22_write(0x12, 0x00);		// temp sensor calibration off
    rf22_write(0x13, 0x00);		// temp sensor offset: 0
    rf22_write(0x1c, 0x04);		// IF filter bandwith: 95,3kHz
    rf22_write(0x1d, 0x40);		// enable AFC
    rf22_write(0x1e, 0x05);		// afc timing
    rf22_write(0x20, 0xC8);		// clock recovery oversampling
    rf22_write(0x21, 0x00);		// clock recovery oversampling2, offset 2
    rf22_write(0x22, 0xA3);		// clock recovery offset 1
    rf22_write(0x23, 0xD7);		// clock recovery offset 0
    rf22_write(0x24, 0x00);		// clock recovery timing 1
    rf22_write(0x25, 0xA6);		// clock recovery timing 0
	rf22_write(0x27, 0x10);		// RSSI Threashold: -120dB
    rf22_write(0x6e, 0x27);		// tx data rate 1
    rf22_write(0x6f, 0x52);		// tx data rate 0
    rf22_write(0x30, 0x8c);		// data access: RX/TX packet handling, enable crc: CCIT
    rf22_write(0x32, 0xff);		// header check enable
    rf22_write(0x33, 0x42);		// 2 word synchronisation
    rf22_write(0x34, 0x10);		// preamble length: 16 nibbles, = 64bits
    rf22_write(0x35, 0x30);		// preamble detection control: 6 nibbles = 24bits
    rf22_write(0x36, 0x2d);		// sync word 3
    rf22_write(0x37, 0xd4);		// sync word 2
    rf22_write(0x38, 0x00);		// sync word 1
    rf22_write(0x39, 0x00);		// sync word 0
    rf22_write(0x3a, 'h');		// transmit header 3
    rf22_write(0x3b, 'o');		// transmit header 2
    rf22_write(0x3c, 'p');		// transmit header 1
    rf22_write(0x3d, 'e');		// transmit header 0
    rf22_write(0x3e, 17);		// packet length
    rf22_write(0x3f, 'h');		// check header 3
    rf22_write(0x40, 'o');		// check header 2
    rf22_write(0x41, 'p');		// check header 1
    rf22_write(0x42, 'e');		// check header 0
    rf22_write(0x43, 0xff);		// header enable mask 3
    rf22_write(0x44, 0xff);		// header enable mask 2
    rf22_write(0x45, 0xff);		// header enable mask 1
    rf22_write(0x46, 0xff);		// header enable mask 0

    rf22_write(0x52, 0x25);		// PA ramp
    rf22_write(0x54, 0x7C);		// preamble detection time

	rf22_write(0x69, 0x20);		// AGC on
    rf22_write(0x6a, 0x0b);		// agc override 2
//	rf22_write(0x6d, 0x07);		// tx power: +13dBm
//  rf22_write(0x6d, 0x02);		// tx power: -2dBm
	rf22_write(0x6d, 0x00);		// tx power: -8dBm

    rf22_write(0x70, 0x00);		// modulation control
    rf22_write(0x71, 0x23);		// modulation control 2: FIFO mode, GFSK
    rf22_write(0x72, 0x48);		// frequency deviation: 45kHz
	rf22_write(0x73, 0x00);		// offset: 0
	rf22_write(0x74, 0x00);		// offset: 0
#ifdef BAND_868
	rf22_write(0x75, 0x73);		// 860-880MHz range
#else
	rf22_write(0x75, 0x53);		// 430-440MHz range
#endif
    rf22_write(0x76, 0x64);		// 434MHz
    rf22_write(0x77, 0x00);

    rf22_write(0x79, 0x0);		// frequency hopping off
    rf22_write(0x7a, 0x0);		// frequency hopping off
}

void rf22_rxmode(void)
{	rf22_write(0x07, 0x01);//	to_ready_mode();
    TX_ANT = 0;
	RX_ANT = 1;
	RXMODE=1;
	
    rf22_read(0x03);			// clear interrupt status
    rf22_read(0x04);			// clear interrupt status

    rf22_write(0x08, 0x02);		// clear RX fifo
    rf22_write(0x08, 0x10);		// clear RX fifo, enable multi packet RX

    rf22_write(0x07, 0x05);		// RX on
}

void rf22_setfreq(unsigned short freq)
{	CONVERTW val;
	val.w=freq;

    rf22_write(0x76, val.b[1]);
    rf22_write(0x77, val.b[0]);
}

void rf22_setbaud(unsigned short bauddiv)
{	CONVERTW val;
	val.w=bauddiv;
	rf22_write(0x6E, val.b[1]);		// set baud high
	rf22_write(0x6F, val.b[0]);		// set baud low
}

void rf22_setrxpara(unsigned char para)
{	if (para>11)
		return;
	rf22_write(0x1C, pgm_read_byte(&baudlist[para].ifbw));    // IF bandwidth
	rf22_write(0x20, pgm_read_byte(&baudlist[para].rxosr1));  // Clock Recovery Oversampling Rate
	rf22_write(0x21, pgm_read_byte(&baudlist[para].rxosr0));  // Clock Recovery Offset 2
	rf22_write(0x22, pgm_read_byte(&baudlist[para].ncoff1));  // Clock Recovery Offset 1
	rf22_write(0x23, pgm_read_byte(&baudlist[para].ncoff0));  // Clock Recovery Offset 0
	rf22_write(0x24, pgm_read_byte(&baudlist[para].crgain1)); // Clock Recovery Timing Loop Gain 1
	rf22_write(0x25, pgm_read_byte(&baudlist[para].crgain0)); // Clock Recovery Timing Loop Gain 0
}

void rf22_setpower(unsigned char power)
{	
	rf22_write(0x6D, power&7);		// set power
}

void rf22_sendpacket(unsigned char *data, unsigned char size)
{	unsigned char i;
	if (size>64)
		size=64;

	rf22_write(0x07, 0x01);		//	switch to ready mode
	RX_ANT = 0;
    TX_ANT = 1;
	RXMODE=0;
	TXLED=1;
    rf22_read(0x03);			// clear interrupt status
    rf22_read(0x04);			// clear interrupt status

    rf22_write(0x08, 0x01);		// clear TX fifo
    rf22_write(0x08, 0x00);		// clear TX fifo
    rf22_write(0x34, 32);		// premable length: 32 nibbles -> 128 Bits
    rf22_write(0x3e, size);		// packet length
    for (i = 0; i<size; i++)
    {    rf22_write(0x7f, *data++);
    }
    rf22_write(0x05, 0x04);		// enable packet send
    rf22_write(0x07, 0x09);		// TX on
    while (nIRQ);
	rf22_write(0x07, 0x01);		//	switch to ready mode
    TX_ANT = 0;
	TXLED=0;
}

unsigned char rf22_getpacket(unsigned char *data)
{	unsigned char i, cnt;
	if (rf22_read(0x02)&32)
		return 0;
	cnt=rf22_read(0x4B);		// packet length
	for (i=0; i<cnt+5; i++)		// Daten
		*data++=rf22_read(0x7f);
	return cnt;
}
